<!DOCTYPE html>
<html lang="en">
  <head>
    <meta charset="UTF-8" />
    <meta name="viewport" content="width=device-width, initial-scale=1.0" />
    <meta http-equiv="X-UA-Compatible" content="ie=edge" />
    <title>{% block title %}{% endblock %}</title>
    <link rel="preconnect" href="https://fonts.googleapis.com" />
    <link rel="preconnect" href="https://fonts.gstatic.com" crossorigin />
    <link
      href="https://fonts.googleapis.com/css2?family=Inter:wght@200;300;400;500;600;700;800;900&display=swap"
      rel="stylesheet"
    />
    <style>
      body {
        font-family: "Inter", sans-serif;
      }
      .table-container {
        overflow: auto;
        width: 90vw;
        height: 700px;
      }
      .styled-table {
        border-collapse: collapse;
        margin: 25px 0;
        font-size: 0.9em;
        font-family: sans-serif;
        min-width: 400px;
        box-shadow: 0 0 20px rgba(0, 0, 0, 0.15);
        width: 100%;
      }

      .styled-table th {
        background-color: #605eff;
        position: sticky;
        font-weight: 700;
        border-right: 1px solid #dddddd;
        color: #ffffff;
        text-align: left;
        top: 0;
        z-index: 2;
      }

      .styled-table tbody tr td:first-child {
        position: sticky;
        left: 0;
        background-color: #e6e4fb;
        font-weight: 700;
      }

      .styled-table th,
      .styled-table td {
        padding: 12px 15px;
        white-space: nowrap;
      }

      .styled-table tbody tr {
        border-bottom: 1px solid #dddddd;
        background-color: #ffffff;
      }

      .styled-table tbody tr:nth-of-type(even) {
        background-color: #e9e9fc;
      }

      .styled-table tbody tr:last-of-type {
        border-bottom: 2px solid #605eff;
      }
    </style>
  </head>
  <body>
    {% block content%} {% endblock %}
  </body>
</html>
